@@ -21,8 +21,9 @@ sys.path.insert(0, os.path.abspath('../..'))
 # extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
 extensions = [
     'sphinx.ext.autodoc',
+    'oslo_config.sphinxconfiggen',
     #'sphinx.ext.intersphinx',
-    'oslosphinx'
+    'openstackdocstheme'
 ]
 
 # autodoc generation is a bit aggressive and a nuisance when doing heavy
@@ -32,6 +33,9 @@ extensions = [
 # The suffix of source filenames.
 source_suffix = '.rst'
 
+config_generator_config_file = '../../etc/networking-hyperv-config-generator.conf'
+sample_config_basename = '_static/networking-hyperv'
+
 # The master toctree document.
 master_doc = 'index'
 
@@ -39,6 +43,11 @@ master_doc = 'index'
 project = 'networking-hyperv'
 copyright = '2013, OpenStack Foundation'
 
+# openstackdocstheme options
+repository_name = 'openstack/networking-hyperv'
+bug_project = 'networking-hyperv'
+bug_tag = ''
+
 # If true, '()' will be appended to :func: etc. cross-reference text.
 add_function_parentheses = True
 
@@ -54,12 +63,14 @@ pygments_style = 'sphinx'
 # The theme to use for HTML and HTML Help pages.  Major themes that come with
 # Sphinx are currently 'default' and 'sphinxdoc'.
 # html_theme_path = ["."]
-# html_theme = '_theme'
-# html_static_path = ['static']
+html_theme = 'openstackdocs'
+html_static_path = ['_static']
 
 # Output file base name for HTML help builder.
 htmlhelp_basename = '%sdoc' % project
 
+html_last_updated_fmt = '%Y-%m-%d %H:%M'
+
 # Grouping the document tree into LaTeX files. List of tuples
 # (source start file, target name, title, author, documentclass
 # [howto/manual]).
