@@ -25,6 +25,7 @@ from neutron.agent import rpc as agent_rpc
 from neutron.common import rpc as n_rpc
 from neutron.common import topics
 from neutron_lib import constants as n_const
+from os_win import constants as os_win_const
 from os_win import exceptions as os_win_exc
 from oslo_concurrency import lockutils
 from oslo_log import log as logging
@@ -32,14 +33,17 @@ from oslo_service import loopingcall
 import six
 
 from networking_hyperv.common.i18n import _, _LI, _LE    # noqa
+from networking_hyperv.neutron import _common_utils as c_util
 from networking_hyperv.neutron.agent import base as base_agent
 from networking_hyperv.neutron import config
 from networking_hyperv.neutron import constants
+from networking_hyperv.neutron import exception
 
 LOG = logging.getLogger(__name__)
 CONF = config.CONF
 
 _synchronized = lockutils.synchronized_with_prefix('n-hv-agent-')
+_port_synchronized = c_util.get_port_synchronized_decorator('n-hv-agent-')
 
 
 class Layer2Agent(base_agent.BaseAgent):
@@ -47,6 +51,8 @@ class Layer2Agent(base_agent.BaseAgent):
     """Contract class for all the layer two agents."""
 
     _AGENT_TOPIC = n_const.L2_AGENT_TOPIC
+    _OVS_EXT_NAME_RE = re.compile(r'.*((open.?v.?switch)|(ovs)).*',
+                                  re.IGNORECASE)
 
     def __init__(self):
         super(Layer2Agent, self).__init__()
@@ -76,8 +82,9 @@ class Layer2Agent(base_agent.BaseAgent):
         self._phys_net_map = agent_config.get(
             'physical_network_vswitch_mappings', [])
         self._local_network_vswitch = agent_config.get(
-            'local_network_vswitch', 'private')
+            'local_network_vswitch')
         self._load_physical_network_mappings(self._phys_net_map)
+        self._validate_vswitches()
 
         self._endpoints.append(self)
         self._event_callback_pairs.extend([
@@ -102,12 +109,12 @@ class Layer2Agent(base_agent.BaseAgent):
             [topics.PORT, topics.DELETE]
         ])
 
-        self.connection = agent_rpc.create_consumers(
+        self._connection = agent_rpc.create_consumers(
             self._endpoints, self._topic, self._consumers,
             start_listening=False
         )
         self._setup_qos_extension()
-        self.connection.consume_in_threads()
+        self._connection.consume_in_threads()
 
         report_interval = CONF.AGENT.report_interval
         if report_interval:
@@ -136,6 +143,49 @@ class Layer2Agent(base_agent.BaseAgent):
                 vswitch = parts[1].strip()
                 self._physical_network_mappings[pattern] = vswitch
 
+    def _validate_vswitches(self):
+        vswitch_names = list(self._physical_network_mappings.values())
+        if self._local_network_vswitch:
+            vswitch_names.append(self._local_network_vswitch)
+
+        vswitches_valid = True
+        for vswitch_name in vswitch_names:
+            try:
+                self._validate_vswitch(vswitch_name)
+            except exception.ValidationError:
+                # We're validating all the vSwitches before erroring out.
+                LOG.error("Validating vSwitch %s failed", vswitch_name)
+                vswitches_valid = False
+
+        # We're currently stopping the service if any of the configured
+        # vSwitches are unavailable.
+        if not vswitches_valid:
+            err_msg = _("Validating one or more configured vSwitches failed.")
+            raise exception.ValidationError(err_msg)
+        elif not vswitch_names:
+            err_msg = _("No vSwitch configured.")
+            raise exception.ValidationError(err_msg)
+
+    def _validate_vswitch(self, vswitch_name):
+        try:
+            vswitch_extensions = self._utils.get_vswitch_extensions(
+                vswitch_name)
+        except os_win_exc.HyperVvSwitchNotFound as exc:
+            raise exception.ValidationError(exc.message)
+
+        for ext in vswitch_extensions:
+            if (self._is_ovs_extension(ext) and
+                    ext['enabled_state'] == os_win_const.CIM_STATE_ENABLED):
+                err_msg = _("The Open vSwitch extension is enabled on the "
+                            "'%s' vSwitch. For this reason, this agent "
+                            "cannot use the specified vSwitch.")
+                raise exception.ValidationError(err_msg % vswitch_name)
+
+    def _is_ovs_extension(self, vswitch_extension):
+        # The OVS extension name keeps changing, while some vendors
+        # redistribute it under a different name.
+        return bool(self._OVS_EXT_NAME_RE.match(vswitch_extension['name']))
+
     def _get_vswitch_name(self, network_type, physical_network):
         """Get the vswitch name for the received network information."""
         if network_type != constants.TYPE_LOCAL:
@@ -143,7 +193,16 @@ class Layer2Agent(base_agent.BaseAgent):
                 physical_network)
         else:
             vswitch_name = self._local_network_vswitch
-        return vswitch_name
+
+        if vswitch_name:
+            return vswitch_name
+
+        err_msg = _("No vSwitch configured for physical network "
+                    "'%(physical_network)s'. Neutron network type: "
+                    "'%(network_type)s'.")
+        raise exception.NetworkingHyperVException(
+            err_msg % dict(physical_network=physical_network,
+                           network_type=network_type))
 
     def _get_vswitch_for_physical_network(self, phys_network_name):
         """Get the vswitch name for the received network name."""
@@ -152,8 +211,6 @@ class Layer2Agent(base_agent.BaseAgent):
                 phys_network_name = ''
             if re.match(pattern, phys_network_name):
                 return self._physical_network_mappings[pattern]
-        # Not found in the mappings, the vswitch has the same name
-        return phys_network_name
 
     def _get_network_vswitch_map_by_port_id(self, port_id):
         """Get the vswitch name for the received port id."""
@@ -192,7 +249,7 @@ class Layer2Agent(base_agent.BaseAgent):
         del self._network_vswitch_map[net_uuid]
 
     def _port_bound(self, port_id, network_id, network_type, physical_network,
-                    segmentation_id):
+                    segmentation_id, port_security_enabled, set_port_sriov):
         """Bind the port to the recived network."""
         LOG.debug("Binding port %s", port_id)
 
@@ -210,6 +267,9 @@ class Layer2Agent(base_agent.BaseAgent):
             vswitch_name=vswitch_map['vswitch_name'],
             switch_port_name=port_id,
         )
+        if set_port_sriov:
+            LOG.debug("Enabling SR-IOV for port: %s", port_id)
+            self._utils.set_vswitch_port_sriov(port_id, True)
 
     def _port_unbound(self, port_id, vnic_deleted=False):
         LOG.debug("Trying to unbind the port %r", port_id)
@@ -229,13 +289,17 @@ class Layer2Agent(base_agent.BaseAgent):
             self._reclaim_local_network(net_uuid)
 
     def _process_added_port(self, device_details):
+        # NOTE(claudiub): A port requiring SR-IOV will specify a PCI slot.
+        set_port_sriov = 'pci_slot' in device_details.get('profile', {})
         self._treat_vif_port(
             port_id=device_details['port_id'],
             network_id=device_details['network_id'],
             network_type=device_details['network_type'],
             physical_network=device_details['physical_network'],
             segmentation_id=device_details['segmentation_id'],
-            admin_state_up=device_details['admin_state_up'])
+            admin_state_up=device_details['admin_state_up'],
+            port_security_enabled=device_details['port_security_enabled'],
+            set_port_sriov=set_port_sriov)
 
     def process_added_port(self, device_details):
         """Process the new ports.
@@ -355,7 +419,8 @@ class Layer2Agent(base_agent.BaseAgent):
             self._utils.update_cache()
             self._refresh_cache = False
 
-        eventlet.spawn_n(self._notify_plugin_on_port_updates)
+        if self._bound_ports or self._unbound_ports:
+            eventlet.spawn_n(self._notify_plugin_on_port_updates)
 
         # notify plugin about port deltas
         if self._added_ports:
@@ -378,6 +443,7 @@ class Layer2Agent(base_agent.BaseAgent):
                 physical_network=physical_network,
                 segmentation_id=segmentation_id,
                 admin_state_up=port['admin_state_up'],
+                port_security_enabled=port['port_security_enabled'],
             )
         else:
             LOG.debug("No port %s defined on agent.", port['id'])
@@ -402,8 +468,14 @@ class Layer2Agent(base_agent.BaseAgent):
         """Provision the network with the received information."""
         pass
 
-    @abc.abstractmethod
+    @_port_synchronized
     def _treat_vif_port(self, port_id, network_id, network_type,
                         physical_network, segmentation_id,
-                        admin_state_up):
-        pass
+                        admin_state_up, port_security_enabled,
+                        set_port_sriov=False):
+        if admin_state_up:
+            self._port_bound(port_id, network_id, network_type,
+                             physical_network, segmentation_id,
+                             port_security_enabled, set_port_sriov)
+        else:
+            self._port_unbound(port_id)
