@@ -24,6 +24,7 @@ import ddt
 import neutron
 from neutron.common import topics
 from neutron.conf.agent import common as neutron_config
+from os_win import constants as os_win_const
 from os_win import exceptions as os_win_exc
 
 from networking_hyperv.neutron.agent import layer2 as agent_base
@@ -47,11 +48,6 @@ class _Layer2Agent(agent_base.Layer2Agent):
                            physical_network, segmentation_id):
         pass
 
-    def _treat_vif_port(self, port_id, network_id, network_type,
-                        physical_network, segmentation_id,
-                        admin_state_up):
-        pass
-
 
 @ddt.ddt
 class TestLayer2Agent(test_base.HyperVBaseTestCase):
@@ -70,21 +66,16 @@ class TestLayer2Agent(test_base.HyperVBaseTestCase):
 
         self._agent = self._get_agent()
 
-        self._agent._qos_ext = mock.MagicMock()
-        self._agent._plugin_rpc = mock.Mock()
-        self._agent._metricsutils = mock.MagicMock()
-        self._agent._utils = mock.MagicMock()
-        self._agent._context = mock.Mock()
-        self._agent._client = mock.MagicMock()
-        self._agent._connection = mock.MagicMock()
-        self._agent._agent_id = mock.Mock()
-        self._agent._utils = mock.MagicMock()
-        self._agent._nvgre_ops = mock.MagicMock()
-        self._agent._vlan_driver = mock.MagicMock()
-        self._agent._physical_network_mappings = collections.OrderedDict()
-        self._agent._config = mock.MagicMock()
+        self._agent._utils = mock.MagicMock(
+            autospec=self._agent._utils)
+        self._agent._plugin_rpc = mock.Mock(
+            autospec=agent_base.agent_rpc.PluginApi)
         self._agent._endpoints = mock.MagicMock()
-        self._agent._event_callback_pairs = mock.MagicMock()
+        self._agent._client = mock.MagicMock(
+            autospec=agent_base.n_rpc.BackingOffClient)
+        self._agent._connection = mock.MagicMock(
+            autospec=agent_base.n_rpc.Connection)
+        self._agent._physical_network_mappings = collections.OrderedDict()
         self._agent._network_vswitch_map = {}
 
     def _get_fake_port_details(self):
@@ -95,18 +86,21 @@ class TestLayer2Agent(test_base.HyperVBaseTestCase):
             'network_type': mock.sentinel.network_type,
             'physical_network': mock.sentinel.physical_network,
             'segmentation_id': mock.sentinel.segmentation_id,
-            'admin_state_up': mock.sentinel.admin_state_up
+            'admin_state_up': mock.sentinel.admin_state_up,
+            'port_security_enabled': mock.sentinel.port_security_enabled,
         }
 
     @mock.patch.object(agent_base.Layer2Agent, '_process_removed_port_event',
                        mock.sentinel._process_removed_port_event)
     @mock.patch.object(agent_base.Layer2Agent, '_process_added_port_event',
                        mock.sentinel._process_added_port_event)
+    @mock.patch.object(agent_base.Layer2Agent, '_validate_vswitches')
     @mock.patch.object(eventlet.tpool, 'set_num_threads')
     @mock.patch.object(agent_base.Layer2Agent,
                        '_load_physical_network_mappings')
     def test_setup(self, mock_load_phys_net_mapp,
-                   mock_set_num_threads):
+                   mock_set_num_threads,
+                   mock_validate_vswitches):
         self.config(
             group="AGENT",
             worker_count=12,
@@ -117,6 +111,7 @@ class TestLayer2Agent(test_base.HyperVBaseTestCase):
         self._agent._setup()
 
         mock_load_phys_net_mapp.assert_called_once_with(["fake_mappings"])
+        mock_validate_vswitches.assert_called_once_with()
         self._agent._endpoints.append.assert_called_once_with(self._agent)
         self.assertIn((self._agent._utils.EVENT_TYPE_CREATE,
                        mock.sentinel._process_added_port_event),
@@ -188,6 +183,86 @@ class TestLayer2Agent(test_base.HyperVBaseTestCase):
             sorted(self._agent._physical_network_mappings.items())
         )
 
+    @ddt.data(True, False)
+    @mock.patch.object(agent_base.Layer2Agent, '_validate_vswitch')
+    def test_validate_vswitches(self, all_valid, mock_validate_vswitch):
+        phys_mappings = {
+            'fakenetwork0': mock.sentinel.vswitch_name_0,
+            'fakenetwork1': mock.sentinel.vswitch_name_1
+        }
+        self._agent._physical_network_mappings = phys_mappings
+        self._agent._local_network_vswitch = mock.sentinel.local_vswitch
+
+        exp_vswitch_names = (
+            list(phys_mappings.values()) + [mock.sentinel.local_vswitch])
+
+        mock_validate_vswitch.side_effect = (
+            exception.ValidationError if not all_valid else None,
+            None, None)
+
+        if all_valid:
+            self._agent._validate_vswitches()
+        else:
+            self.assertRaises(exception.ValidationError,
+                              self._agent._validate_vswitches)
+
+        mock_validate_vswitch.assert_has_calls(
+            [mock.call(vswitch_name)
+             for vswitch_name in exp_vswitch_names],
+            any_order=True)
+
+    def test_validate_vswitches_none_configured(self):
+        self._agent._physical_network_mappings = {}
+        self._agent._local_network_vswitch = None
+
+        self.assertRaises(exception.ValidationError,
+                          self._agent._validate_vswitches)
+
+    def test_is_ovs_extension(self):
+        valid_ovs_ext_names = ['Cloudbase Open vSwitch Extension',
+                               'Open vSwitch Extension',
+                               'VendorX ovs',
+                               'open v-switch',
+                               'openvswitch']
+        for ovs_ext_name in valid_ovs_ext_names:
+            ext = dict(name=ovs_ext_name)
+            self.assertTrue(
+                self._agent._is_ovs_extension(ext))
+
+        ext = dict(name='fake extension')
+        self.assertFalse(self._agent._is_ovs_extension(ext))
+
+    @ddt.data({},
+              {'is_valid': False, 'exists': False},
+              {'is_valid': False,
+               'is_ovs_ext': True,
+               'enabled_state': os_win_const.CIM_STATE_ENABLED},
+              {'is_ovs_ext': True,
+               'enabled_state': os_win_const.CIM_STATE_DISABLED},
+              {'enabled_state': os_win_const.CIM_STATE_ENABLED})
+    @ddt.unpack
+    @mock.patch.object(agent_base.Layer2Agent, '_is_ovs_extension')
+    def test_validate_vswitch(self, mock_is_ovs_ext,
+                              is_valid=True, exists=True, is_ovs_ext=False,
+                              enabled_state=os_win_const.CIM_STATE_ENABLED):
+        extension = dict(enabled_state=enabled_state)
+        mock_is_ovs_ext.return_value = is_ovs_ext
+
+        mock_get_ext = self._agent._utils.get_vswitch_extensions
+        mock_get_ext.side_effect = (
+            [[extension]] if exists
+            else os_win_exc.HyperVvSwitchNotFound(message='fake_msg'))
+
+        if is_valid:
+            self._agent._validate_vswitch(mock.sentinel.vswitch_name)
+            mock_is_ovs_ext.assert_called_once_with(extension)
+        else:
+            self.assertRaises(exception.ValidationError,
+                              self._agent._validate_vswitch,
+                              mock.sentinel.vswitch_name)
+
+        mock_get_ext.assert_called_once_with(mock.sentinel.vswitch_name)
+
     def test_get_vswitch_for_physical_network_with_default_switch(self):
         test_mappings = [
             'fakenetwork:fake_vswitch',
@@ -202,7 +277,7 @@ class TestLayer2Agent(test_base.HyperVBaseTestCase):
         self.assertEqual('fake_vswitch_2', get_vswitch('fakenetwork2$'))
         self.assertEqual('fake_vswitch_3', get_vswitch('fakenetwork3'))
         self.assertEqual('fake_vswitch_3', get_vswitch('fakenetwork35'))
-        self.assertEqual('fake_network1', get_vswitch('fake_network1'))
+        self.assertIsNone(get_vswitch('fake_network1'))
 
     def test_get_vswitch_for_physical_network_without_default_switch(self):
         test_mappings = [
@@ -223,7 +298,7 @@ class TestLayer2Agent(test_base.HyperVBaseTestCase):
             'fakenetwork2:fake_vswitch_2'
         ]
         self._agent._load_physical_network_mappings(test_mappings)
-        self.assertEqual('', get_vswitch(None))
+        self.assertIsNone(get_vswitch(None))
 
         test_mappings = [
             'fakenetwork:fake_vswitch',
@@ -250,6 +325,19 @@ class TestLayer2Agent(test_base.HyperVBaseTestCase):
         mock_get_vswitch_for_phys_net.assert_called_once_with(
             mock.sentinel.FAKE_PHYSICAL_NETWORK)
 
+    @mock.patch.object(agent_base.Layer2Agent,
+                       "_get_vswitch_for_physical_network")
+    @ddt.data(constants.TYPE_VLAN, constants.TYPE_LOCAL)
+    def test_get_vswitch_name_missing(self, network_type,
+                                      mock_get_vswitch_for_phys_net):
+        mock_get_vswitch_for_phys_net.return_value = None
+        self._agent._local_network_vswitch = ''
+
+        self.assertRaises(exception.NetworkingHyperVException,
+                          self._agent._get_vswitch_name,
+                          network_type,
+                          mock.sentinel.FAKE_PHYSICAL_NETWORK)
+
     def test_get_network_vswitch_map_by_port_id(self):
         net_uuid = 'net-uuid'
         self._agent._network_vswitch_map = {
@@ -319,21 +407,8 @@ class TestLayer2Agent(test_base.HyperVBaseTestCase):
         self.assertNotIn(mock.sentinel.net_id,
                          self._agent._network_vswitch_map)
 
-    def test_port_bound_no_metrics(self):
-        self._agent.enable_metrics_collection = False
-        port = mock.sentinel.port
-        net_uuid = 'my-net-uuid'
-        self._agent._network_vswitch_map[net_uuid] = {
-            'ports': [],
-            'vswitch_name': []
-        }
-        self._agent._port_bound(str(port), net_uuid, 'vlan', None, None)
-
-        self.assertFalse(self._agent._utils.add_metrics_collection_acls.called)
-
-    @mock.patch.object(agent_base.Layer2Agent,
-                       '_provision_network')
-    def _check_port_bound_net_type(self, mock_provision_network, network_type):
+    @mock.patch.object(_Layer2Agent, '_provision_network')
+    def test_port_bound_net_type(self, mock_provision_network):
         net_uuid = 'my-net-uuid'
         fake_map = {'vswitch_name': mock.sentinel.vswitch_name,
                     'ports': []}
@@ -344,16 +419,20 @@ class TestLayer2Agent(test_base.HyperVBaseTestCase):
         mock_provision_network.side_effect = fake_prov_network
 
         self._agent._port_bound(mock.sentinel.port_id,
-                                net_uuid, network_type,
+                                net_uuid, mock.sentinel.network_type,
                                 mock.sentinel.physical_network,
-                                mock.sentinel.segmentation_id)
+                                mock.sentinel.segmentation_id,
+                                mock.sentinel.port_security_enabled, True)
 
         self.assertIn(mock.sentinel.port_id, fake_map['ports'])
         mock_provision_network.assert_called_once_with(
-            mock.sentinel.port_id, net_uuid, network_type,
+            mock.sentinel.port_id, net_uuid, mock.sentinel.network_type,
             mock.sentinel.physical_network, mock.sentinel.segmentation_id)
         self._agent._utils.connect_vnic_to_vswitch.assert_called_once_with(
-            mock.sentinel.vswitch_name, mock.sentinel.port_id)
+            vswitch_name=mock.sentinel.vswitch_name,
+            switch_port_name=mock.sentinel.port_id)
+        self._agent._utils.set_vswitch_port_sriov.assert_called_once_with(
+            mock.sentinel.port_id, True)
 
     @mock.patch.object(agent_base.Layer2Agent,
                        '_get_network_vswitch_map_by_port_id')
@@ -389,6 +468,17 @@ class TestLayer2Agent(test_base.HyperVBaseTestCase):
     def test_port_unbound_port_not_found(self):
         self._check_port_unbound()
 
+    @mock.patch.object(_Layer2Agent, '_treat_vif_port')
+    def test_process_added_port_sriov(self, mock_treat_vif_port):
+        details = self._get_fake_port_details()
+        details.pop('device')
+        port_details = dict(profile={'pci_slot': mock.sentinel.pci_slot},
+                            **self._get_fake_port_details())
+
+        self._agent.process_added_port(port_details)
+        mock_treat_vif_port.assert_called_once_with(set_port_sriov=True,
+                                                    **details)
+
     @ddt.data(os_win_exc.HyperVvNicNotFound(vnic_name='fake_vnic'),
               os_win_exc.HyperVPortNotFoundException(port_name='fake_port'),
               Exception)
@@ -397,14 +487,19 @@ class TestLayer2Agent(test_base.HyperVBaseTestCase):
         mock_treat_vif_port.side_effect = side_effect
         self._agent._added_ports = set()
         details = self._get_fake_port_details()
+        details.pop('device')
+        port_details = self._get_fake_port_details()
 
-        self._agent.process_added_port(details)
+        self._agent.process_added_port(port_details)
 
         if isinstance(side_effect, os_win_exc.HyperVvNicNotFound):
             self.assertNotIn(mock.sentinel.device, self._agent._added_ports)
         else:
             self.assertIn(mock.sentinel.device, self._agent._added_ports)
 
+        mock_treat_vif_port.assert_called_once_with(set_port_sriov=False,
+                                                    **details)
+
     def test_treat_devices_added_returns_true_for_missing_device(self):
         self._agent._added_ports = set([mock.sentinel.port_id])
         attrs = {'get_devices_details_list.side_effect': Exception()}
@@ -510,6 +605,7 @@ class TestLayer2Agent(test_base.HyperVBaseTestCase):
     def test_work(self, mock_spawn, mock_treat_dev_added,
                   mock_treat_dev_removed):
         self._agent._refresh_cache = True
+        self._agent._bound_ports = set([mock.sentinel.bound_port])
         self._agent._added_ports = set([mock.sentinel.bound_port])
         self._agent._removed_ports = set([mock.sentinel.unbound_port])
 
@@ -522,16 +618,29 @@ class TestLayer2Agent(test_base.HyperVBaseTestCase):
         mock_treat_dev_added.assert_called_once_with()
         mock_treat_dev_removed.assert_called_once_with()
 
+    @mock.patch.object(agent_base.Layer2Agent, '_treat_devices_removed')
+    @mock.patch.object(agent_base.Layer2Agent, '_treat_devices_added')
+    @mock.patch('eventlet.spawn_n')
+    def test_work_noop(self, mock_spawn, mock_treat_dev_added,
+                       mock_treat_dev_removed):
+        self._agent._work()
+
+        self.assertFalse(mock_spawn.called)
+        self.assertFalse(mock_treat_dev_added.called)
+        self.assertFalse(mock_treat_dev_removed.called)
+
     def test_port_update_not_found(self):
         self._agent._utils.vnic_port_exists.return_value = False
         port = {'id': mock.sentinel.port_id}
         self._agent.port_update(self._agent._context, port)
 
-    def test_port_update(self):
+    @mock.patch.object(agent_base.Layer2Agent, '_treat_vif_port')
+    def test_port_update(self, mock_treat_vif_port):
         self._agent._utils.vnic_port_exists.return_value = True
         port = {'id': mock.sentinel.port_id,
                 'network_id': mock.sentinel.network_id,
-                'admin_state_up': mock.sentinel.admin_state_up}
+                'admin_state_up': mock.sentinel.admin_state_up,
+                'port_security_enabled': mock.sentinel.port_security_enabled}
 
         self._agent.port_update(self._agent._context, port,
                                 mock.sentinel.network_type,
@@ -554,3 +663,27 @@ class TestLayer2Agent(test_base.HyperVBaseTestCase):
     def test_network_delete_not_defined(self, mock_reclaim_local_network):
         self._agent.network_delete(mock.sentinel.context, mock.sentinel.net_id)
         self.assertFalse(mock_reclaim_local_network.called)
+
+    @mock.patch.object(agent_base.Layer2Agent, '_port_bound')
+    def test_treat_vif_port_state_up(self, mock_port_bound):
+        self._agent._treat_vif_port(
+            mock.sentinel.port_id, mock.sentinel.network_id,
+            mock.sentinel.network_type, mock.sentinel.physical_network,
+            mock.sentinel.segmentation_id, True,
+            mock.sentinel.port_security_enabled)
+
+        mock_port_bound.assert_called_once_with(
+            mock.sentinel.port_id, mock.sentinel.network_id,
+            mock.sentinel.network_type, mock.sentinel.physical_network,
+            mock.sentinel.segmentation_id, mock.sentinel.port_security_enabled,
+            False)
+
+    @mock.patch.object(agent_base.Layer2Agent, '_port_unbound')
+    def test_treat_vif_port_state_down(self, mock_port_unbound):
+        self._agent._treat_vif_port(
+            mock.sentinel.port_id, mock.sentinel.network_id,
+            mock.sentinel.network_type, mock.sentinel.physical_network,
+            mock.sentinel.segmentation_id, False,
+            mock.sentinel.port_security_enabled)
+
+        mock_port_unbound.assert_called_once_with(mock.sentinel.port_id)
